@@ -1,6 +1,7 @@
 package e2e
 
 import (
+	"encoding/json"
 	"fmt"
 	"strings"
 	"testing"
@@ -19,13 +20,12 @@ import (
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/tools/cache"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/ownerutil"
-
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1"
+	v1 "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/install"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/ownerutil"
 )
 
 func checkOperatorGroupAnnotations(obj metav1.Object, op *v1.OperatorGroup, checkTargetNamespaces bool, targetNamespaces string) error {
@@ -139,11 +139,6 @@ func TestOperatorGroup(t *testing.T) {
 	}
 	_, err = crc.OperatorsV1().OperatorGroups(opGroupNamespace).Create(&operatorGroup)
 	require.NoError(t, err)
-	defer func() {
-		err = crc.OperatorsV1().OperatorGroups(opGroupNamespace).Delete(operatorGroup.Name, &metav1.DeleteOptions{})
-		require.NoError(t, err)
-	}()
-
 	expectedOperatorGroupStatus := v1.OperatorGroupStatus{
 		Namespaces: []string{opGroupNamespace, createdOtherNamespace.GetName()},
 	}
@@ -407,6 +402,36 @@ func TestOperatorGroup(t *testing.T) {
 		return err
 	})
 	require.NoError(t, err)
+
+	err = crc.OperatorsV1().OperatorGroups(opGroupNamespace).Delete(operatorGroup.Name, &metav1.DeleteOptions{})
+	require.NoError(t, err)
+	t.Log("Waiting for OperatorGroup RBAC to be garbage collected")
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		_, err := c.KubernetesInterface().RbacV1().ClusterRoles().Get(operatorGroup.Name+"-admin", metav1.GetOptions{})
+		if err == nil {
+			return false, nil
+		}
+		return true, err
+	})
+	require.True(t, errors.IsNotFound(err))
+
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		_, err := c.KubernetesInterface().RbacV1().ClusterRoles().Get(operatorGroup.Name+"-edit", metav1.GetOptions{})
+		if err == nil {
+			return false, nil
+		}
+		return true, err
+	})
+	require.True(t, errors.IsNotFound(err))
+
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		_, err := c.KubernetesInterface().RbacV1().ClusterRoles().Get(operatorGroup.Name+"-view", metav1.GetOptions{})
+		if err == nil {
+			return false, nil
+		}
+		return true, err
+	})
+	require.True(t, errors.IsNotFound(err))
 }
 
 func createProjectAdmin(t *testing.T, c operatorclient.ClientInterface, namespace string) (string, cleanupFunc) {
@@ -496,10 +521,82 @@ func TestOperatorGroupRoleAggregation(t *testing.T) {
 	_, err = fetchCSV(t, crc, csvA.GetName(), nsA, csvSucceededChecker)
 	require.NoError(t, err)
 
+	// Create a csv for an apiserver
+	depName := genName("hat-server")
+	mockGroup := fmt.Sprintf("hats.%s.redhat.com", genName(""))
+	version := "v1alpha1"
+	mockGroupVersion := strings.Join([]string{mockGroup, version}, "/")
+	mockKinds := []string{"fez", "fedora"}
+	mockNames := []string{"fezs", "fedoras"}
+	depSpec := newMockExtServerDeployment(depName, mockGroupVersion, mockKinds)
+	strategy := install.StrategyDetailsDeployment{
+		DeploymentSpecs: []install.StrategyDeploymentSpec{
+			{
+				Name: depName,
+				Spec: depSpec,
+			},
+		},
+	}
+	strategyRaw, err := json.Marshal(strategy)
+	owned := make([]v1alpha1.APIServiceDescription, len(mockKinds))
+	for i, kind := range mockKinds {
+		owned[i] = v1alpha1.APIServiceDescription{
+			Name:           mockNames[i],
+			Group:          mockGroup,
+			Version:        version,
+			Kind:           kind,
+			DeploymentName: depName,
+			ContainerPort:  int32(5443),
+			DisplayName:    kind,
+			Description:    fmt.Sprintf("A %s", kind),
+		}
+	}
+
+	csvB := v1alpha1.ClusterServiceVersion{
+		Spec: v1alpha1.ClusterServiceVersionSpec{
+			MinKubeVersion: "0.0.0",
+			InstallModes: []v1alpha1.InstallMode{
+				{
+					Type:      v1alpha1.InstallModeTypeOwnNamespace,
+					Supported: true,
+				},
+				{
+					Type:      v1alpha1.InstallModeTypeSingleNamespace,
+					Supported: true,
+				},
+				{
+					Type:      v1alpha1.InstallModeTypeMultiNamespace,
+					Supported: true,
+				},
+				{
+					Type:      v1alpha1.InstallModeTypeAllNamespaces,
+					Supported: true,
+				},
+			},
+			InstallStrategy: v1alpha1.NamedInstallStrategy{
+				StrategyName:    install.InstallStrategyNameDeployment,
+				StrategySpecRaw: strategyRaw,
+			},
+			APIServiceDefinitions: v1alpha1.APIServiceDefinitions{
+				Owned: owned,
+			},
+		},
+	}
+	csvB.SetName(depName)
+
+	// Create the APIService CSV
+	cleanupCSV, err := createCSV(t, c, crc, csvB, nsA, false, true)
+	require.NoError(t, err)
+	defer cleanupCSV()
+
+	_, err = fetchCSV(t, crc, csvB.GetName(), nsA, csvSucceededChecker)
+	require.NoError(t, err)
+
 	// Ensure clusterroles created and aggregated for access provided APIs
 	padmin, cleanupPadmin := createProjectAdmin(t, c, nsA)
 	defer cleanupPadmin()
 
+	// Check CRD access aggregated
 	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
 		res, err := c.KubernetesInterface().AuthorizationV1().SubjectAccessReviews().Create(&authorizationv1.SubjectAccessReview{
 			Spec: authorizationv1.SubjectAccessReviewSpec{
@@ -523,6 +620,31 @@ func TestOperatorGroupRoleAggregation(t *testing.T) {
 		return res.Status.Allowed, nil
 	})
 	require.NoError(t, err)
+
+	// Check apiserver access aggregated
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		res, err := c.KubernetesInterface().AuthorizationV1().SubjectAccessReviews().Create(&authorizationv1.SubjectAccessReview{
+			Spec: authorizationv1.SubjectAccessReviewSpec{
+				User: padmin,
+				ResourceAttributes: &authorizationv1.ResourceAttributes{
+					Namespace: nsA,
+					Group:     mockGroup,
+					Version:   version,
+					Resource:  mockNames[1],
+					Verb:      "create",
+				},
+			},
+		})
+		if err != nil {
+			return false, err
+		}
+		if res == nil {
+			return false, nil
+		}
+		t.Logf("checking padmin for permission: %#v", res)
+		return res.Status.Allowed, nil
+	})
+	require.NoError(t, err)
 }
 
 func TestOperatorGroupInstallModeSupport(t *testing.T) {
@@ -1123,8 +1245,7 @@ func TestStaticProviderOperatorGroup(t *testing.T) {
 	}
 
 	// Create subscription for csvA in namespaceB
-	subAName := genName("a-" +
-		"")
+	subAName := genName("a-")
 	cleanupSubA := createSubscriptionForCatalog(t, crc, nsB, subAName, catalog, pkgA, stableChannel, pkgAStable, v1alpha1.ApprovalAutomatic)
 	defer cleanupSubA()
 	subA, err := fetchSubscription(t, crc, nsB, subAName, subscriptionHasInstallPlanChecker)
@@ -1492,3 +1613,240 @@ func TestCSVCopyWatchingAllNamespaces(t *testing.T) {
 	})
 	require.NoError(t, err)
 }
+
+func TestOperatorGroupInsufficientPermissionsResolveViaRBAC(t *testing.T) {
+	defer cleaner.NotifyTestComplete(t, true)
+
+	log := func(s string) {
+		t.Logf("%s: %s", time.Now().Format("15:04:05.9999"), s)
+	}
+
+	c := newKubeClient(t)
+	crc := newCRClient(t)
+	csvName := genName("another-csv-")
+
+	newNamespaceName := genName(testNamespace + "-")
+
+	_, err := c.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: newNamespaceName,
+		},
+	})
+	require.NoError(t, err)
+	defer func() {
+		err = c.KubernetesInterface().CoreV1().Namespaces().Delete(newNamespaceName, &metav1.DeleteOptions{})
+		require.NoError(t, err)
+	}()
+
+	log("Creating CRD")
+	mainCRDPlural := genName("opgroup")
+	mainCRD := newCRD(mainCRDPlural)
+	cleanupCRD, err := createCRD(c, mainCRD)
+	require.NoError(t, err)
+	defer cleanupCRD()
+
+	log("Creating operator group")
+	serviceAccountName := genName("nginx-sa")
+	// intentionally creating an operator group without a service account already existing
+	operatorGroup := v1.OperatorGroup{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      genName("e2e-operator-group-"),
+			Namespace: newNamespaceName,
+		},
+		Spec: v1.OperatorGroupSpec{
+			ServiceAccountName: serviceAccountName,
+			TargetNamespaces:   []string{newNamespaceName},
+		},
+	}
+	_, err = crc.OperatorsV1().OperatorGroups(newNamespaceName).Create(&operatorGroup)
+	require.NoError(t, err)
+
+	log("Creating CSV")
+
+	// Create a new NamedInstallStrategy
+	deploymentName := genName("operator-deployment")
+	namedStrategy := newNginxInstallStrategy(deploymentName, nil, nil)
+
+	aCSV := newCSV(csvName, newNamespaceName, "", semver.MustParse("0.0.0"), []apiextensions.CustomResourceDefinition{mainCRD}, nil, namedStrategy)
+	createdCSV, err := crc.OperatorsV1alpha1().ClusterServiceVersions(newNamespaceName).Create(&aCSV)
+	require.NoError(t, err)
+
+	serviceAccount := &corev1.ServiceAccount{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: newNamespaceName,
+			Name:      serviceAccountName,
+		},
+	}
+	ownerutil.AddNonBlockingOwner(serviceAccount, createdCSV)
+	err = ownerutil.AddOwnerLabels(serviceAccount, createdCSV)
+	require.NoError(t, err)
+
+	_, err = c.CreateServiceAccount(serviceAccount)
+	require.NoError(t, err)
+
+	log("wait for CSV to fail")
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		fetched, err := crc.OperatorsV1alpha1().ClusterServiceVersions(newNamespaceName).Get(createdCSV.GetName(), metav1.GetOptions{})
+		if err != nil {
+			return false, err
+		}
+		log(fmt.Sprintf("%s (%s): %s", fetched.Status.Phase, fetched.Status.Reason, fetched.Status.Message))
+		return csvFailedChecker(fetched), nil
+	})
+	require.NoError(t, err)
+
+	// now add cluster admin permissions to service account
+	role := &rbacv1.Role{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: newNamespaceName,
+			Name:      serviceAccountName + "-role",
+		},
+		Rules: []rbacv1.PolicyRule{
+			{
+				Verbs:     []string{"*"},
+				APIGroups: []string{"*"},
+				Resources: []string{"*"},
+			},
+		},
+	}
+	ownerutil.AddNonBlockingOwner(role, createdCSV)
+	err = ownerutil.AddOwnerLabels(role, createdCSV)
+	require.NoError(t, err)
+
+	roleBinding := &rbacv1.RoleBinding{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: newNamespaceName,
+			Name:      serviceAccountName + "-rb",
+		},
+		Subjects: []rbacv1.Subject{
+			{
+				Kind:      "ServiceAccount",
+				Name:      serviceAccountName,
+				Namespace: newNamespaceName,
+			},
+		},
+		RoleRef: rbacv1.RoleRef{
+			Kind: "Role",
+			Name: role.GetName(),
+		},
+	}
+	ownerutil.AddNonBlockingOwner(roleBinding, createdCSV)
+	err = ownerutil.AddOwnerLabels(roleBinding, createdCSV)
+	require.NoError(t, err)
+
+	_, err = c.CreateRole(role)
+	require.NoError(t, err)
+	_, err = c.CreateRoleBinding(roleBinding)
+	require.NoError(t, err)
+
+	log("wait for CSV to succeeed")
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		fetched, err := crc.OperatorsV1alpha1().ClusterServiceVersions(newNamespaceName).Get(createdCSV.GetName(), metav1.GetOptions{})
+		if err != nil {
+			return false, err
+		}
+		log(fmt.Sprintf("%s (%s): %s", fetched.Status.Phase, fetched.Status.Reason, fetched.Status.Message))
+		return csvSucceededChecker(fetched), nil
+	})
+	require.NoError(t, err)
+}
+
+func TestOperatorGroupInsufficientPermissionsResolveViaServiceAccountRemoval(t *testing.T) {
+	defer cleaner.NotifyTestComplete(t, true)
+
+	log := func(s string) {
+		t.Logf("%s: %s", time.Now().Format("15:04:05.9999"), s)
+	}
+
+	c := newKubeClient(t)
+	crc := newCRClient(t)
+	csvName := genName("another-csv-")
+
+	newNamespaceName := genName(testNamespace + "-")
+
+	_, err := c.KubernetesInterface().CoreV1().Namespaces().Create(&corev1.Namespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: newNamespaceName,
+		},
+	})
+	require.NoError(t, err)
+	defer func() {
+		err = c.KubernetesInterface().CoreV1().Namespaces().Delete(newNamespaceName, &metav1.DeleteOptions{})
+		require.NoError(t, err)
+	}()
+
+	log("Creating CRD")
+	mainCRDPlural := genName("opgroup")
+	mainCRD := newCRD(mainCRDPlural)
+	cleanupCRD, err := createCRD(c, mainCRD)
+	require.NoError(t, err)
+	defer cleanupCRD()
+
+	log("Creating operator group")
+	serviceAccountName := genName("nginx-sa")
+	// intentionally creating an operator group without a service account already existing
+	operatorGroup := v1.OperatorGroup{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      genName("e2e-operator-group-"),
+			Namespace: newNamespaceName,
+		},
+		Spec: v1.OperatorGroupSpec{
+			ServiceAccountName: serviceAccountName,
+			TargetNamespaces:   []string{newNamespaceName},
+		},
+	}
+	_, err = crc.OperatorsV1().OperatorGroups(newNamespaceName).Create(&operatorGroup)
+	require.NoError(t, err)
+
+	log("Creating CSV")
+
+	// Create a new NamedInstallStrategy
+	deploymentName := genName("operator-deployment")
+	namedStrategy := newNginxInstallStrategy(deploymentName, nil, nil)
+
+	aCSV := newCSV(csvName, newNamespaceName, "", semver.MustParse("0.0.0"), []apiextensions.CustomResourceDefinition{mainCRD}, nil, namedStrategy)
+	createdCSV, err := crc.OperatorsV1alpha1().ClusterServiceVersions(newNamespaceName).Create(&aCSV)
+	require.NoError(t, err)
+
+	serviceAccount := &corev1.ServiceAccount{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: newNamespaceName,
+			Name:      serviceAccountName,
+		},
+	}
+	ownerutil.AddNonBlockingOwner(serviceAccount, createdCSV)
+	err = ownerutil.AddOwnerLabels(serviceAccount, createdCSV)
+	require.NoError(t, err)
+
+	_, err = c.CreateServiceAccount(serviceAccount)
+	require.NoError(t, err)
+
+	log("wait for CSV to fail")
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		fetched, err := crc.OperatorsV1alpha1().ClusterServiceVersions(newNamespaceName).Get(createdCSV.GetName(), metav1.GetOptions{})
+		if err != nil {
+			return false, err
+		}
+		log(fmt.Sprintf("%s (%s): %s", fetched.Status.Phase, fetched.Status.Reason, fetched.Status.Message))
+		return csvFailedChecker(fetched), nil
+	})
+	require.NoError(t, err)
+
+	// now remove operator group specified service account
+	createdOpGroup, err := crc.OperatorsV1().OperatorGroups(newNamespaceName).Get(operatorGroup.GetName(), metav1.GetOptions{})
+	require.NoError(t, err)
+	createdOpGroup.Spec.ServiceAccountName = ""
+	_, err = crc.OperatorsV1().OperatorGroups(newNamespaceName).Update(createdOpGroup)
+	require.NoError(t, err)
+
+	log("wait for CSV to succeeed")
+	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		fetched, err := crc.OperatorsV1alpha1().ClusterServiceVersions(newNamespaceName).Get(createdCSV.GetName(), metav1.GetOptions{})
+		if err != nil {
+			return false, err
+		}
+		log(fmt.Sprintf("%s (%s): %s", fetched.Status.Phase, fetched.Status.Reason, fetched.Status.Message))
+		return csvSucceededChecker(fetched), nil
+	})
+	require.NoError(t, err)
+}
